@@ -15,7 +15,7 @@
 // REQUIRES: stable-runtime
 
 // Stack histories currently are not recorded on x86.
-// XFAIL: x86_64
+// XFAIL: target=x86_64{{.*}}
 
 void USE(void *x) { // pretend_to_do_something(void *x)
   __asm__ __volatile__(""
@@ -40,7 +40,7 @@ __attribute__((noinline)) void Unrelated3() {
 __attribute__((noinline)) char buggy() {
   char *volatile p;
   {
-    char zzz[0x1000];
+    char zzz[0x1000] = {};
     p = zzz;
   }
   return *p;
