@@ -14,8 +14,10 @@
 #include "flang/Evaluate/expression.h"
 #include "flang/Evaluate/fold.h"
 #include "flang/Evaluate/tools.h"
-#include "flang/Lower/IntrinsicCall.h"
+#include "flang/Lower/StatementContext.h"
+#include "flang/Optimizer/Builder/IntrinsicCall.h"
 #include "flang/Optimizer/Builder/Todo.h"
+#include "flang/Semantics/tools.h"
 #include <optional>
 
 /// Is this a call to MIN or MAX intrinsic with arguments that may be absent at
@@ -53,6 +55,30 @@ static bool isIshftcWithDynamicallyOptionalArg(
          Fortran::evaluate::MayBePassedAsAbsentOptional(*expr, foldingContext);
 }
 
+/// Is this a call to ASSOCIATED where the TARGET is an OPTIONAL (but not a
+/// deallocated allocatable or disassociated pointer)?
+/// Subtle: contrary to other intrinsic optional arguments, disassociated
+/// POINTER and unallocated ALLOCATABLE actual argument are not considered
+/// absent here. This is because ASSOCIATED has special requirements for TARGET
+/// actual arguments that are POINTERs. There is no precise requirements for
+/// ALLOCATABLEs, but all existing Fortran compilers treat them similarly to
+/// POINTERs. That is: unallocated TARGETs cause ASSOCIATED to rerun false.  The
+/// runtime deals with the disassociated/unallocated case. Simply ensures that
+/// TARGET that are OPTIONAL get conditionally emboxed here to convey the
+/// optional aspect to the runtime.
+static bool isAssociatedWithDynamicallyOptionalArg(
+    llvm::StringRef name, const Fortran::evaluate::ProcedureRef &procRef,
+    Fortran::evaluate::FoldingContext &foldingContext) {
+  if (name != "associated" || procRef.arguments().size() < 2)
+    return false;
+  auto *expr = Fortran::evaluate::UnwrapExpr<Fortran::lower::SomeExpr>(
+      procRef.arguments()[1]);
+  const Fortran::semantics::Symbol *sym{
+      expr ? Fortran::evaluate::UnwrapWholeSymbolOrComponentDataRef(expr)
+           : nullptr};
+  return (sym && Fortran::semantics::IsOptional(*sym));
+}
+
 bool Fortran::lower::intrinsicRequiresCustomOptionalHandling(
     const Fortran::evaluate::ProcedureRef &procRef,
     const Fortran::evaluate::SpecificIntrinsic &intrinsic,
@@ -60,7 +86,30 @@ bool Fortran::lower::intrinsicRequiresCustomOptionalHandling(
   llvm::StringRef name = intrinsic.name;
   Fortran::evaluate::FoldingContext &fldCtx = converter.getFoldingContext();
   return isMinOrMaxWithDynamicallyOptionalArg(name, procRef, fldCtx) ||
-         isIshftcWithDynamicallyOptionalArg(name, procRef, fldCtx);
+         isIshftcWithDynamicallyOptionalArg(name, procRef, fldCtx) ||
+         isAssociatedWithDynamicallyOptionalArg(name, procRef, fldCtx);
+}
+
+/// Generate the FIR+MLIR operations for the generic intrinsic \p name
+/// with arguments \p args and the expected result type \p resultType.
+/// Returned fir::ExtendedValue is the returned Fortran intrinsic value.
+fir::ExtendedValue
+Fortran::lower::genIntrinsicCall(fir::FirOpBuilder &builder, mlir::Location loc,
+                                 llvm::StringRef name,
+                                 std::optional<mlir::Type> resultType,
+                                 llvm::ArrayRef<fir::ExtendedValue> args,
+                                 Fortran::lower::StatementContext &stmtCtx) {
+  auto [result, mustBeFreed] =
+      fir::genIntrinsicCall(builder, loc, name, resultType, args);
+  if (mustBeFreed) {
+    mlir::Value addr = fir::getBase(result);
+    if (auto *box = result.getBoxOf<fir::BoxValue>())
+      addr =
+          builder.create<fir::BoxAddrOp>(loc, box->getMemTy(), box->getAddr());
+    fir::FirOpBuilder *bldr = &builder;
+    stmtCtx.attachCleanup([=]() { bldr->create<fir::FreeMemOp>(loc, addr); });
+  }
+  return result;
 }
 
 static void prepareMinOrMaxArguments(
@@ -68,7 +117,7 @@ static void prepareMinOrMaxArguments(
     const Fortran::evaluate::SpecificIntrinsic &intrinsic,
     std::optional<mlir::Type> retTy,
     const Fortran::lower::OperandPrepare &prepareOptionalArgument,
-    const Fortran::lower::OperandPrepare &prepareOtherArgument,
+    const Fortran::lower::OperandPrepareAs &prepareOtherArgument,
     Fortran::lower::AbstractConverter &converter) {
   assert(retTy && "MIN and MAX must have a return type");
   mlir::Type resultType = *retTy;
@@ -83,7 +132,7 @@ static void prepareMinOrMaxArguments(
     if (arg.index() <= 1 || !Fortran::evaluate::MayBePassedAsAbsentOptional(
                                 *expr, converter.getFoldingContext())) {
       // Non optional arguments.
-      prepareOtherArgument(*expr);
+      prepareOtherArgument(*expr, fir::LowerIntrinsicArgAs::Value);
     } else {
       // Dynamically optional arguments.
       // Subtle: even for scalar the if-then-else will be generated in the loop
@@ -106,10 +155,11 @@ lowerMinOrMax(fir::FirOpBuilder &builder, mlir::Location loc,
   assert(retTy && "MIN and MAX must have a return type");
   mlir::Type resultType = *retTy;
   llvm::SmallVector<fir::ExtendedValue> args;
-  args.push_back(getOperand(0));
-  args.push_back(getOperand(1));
-  mlir::Value extremum = fir::getBase(Fortran::lower::genIntrinsicCall(
-      builder, loc, name, resultType, args, stmtCtx));
+  const bool loadOperand = true;
+  args.push_back(getOperand(0, loadOperand));
+  args.push_back(getOperand(1, loadOperand));
+  mlir::Value extremum = fir::getBase(
+      genIntrinsicCall(builder, loc, name, resultType, args, stmtCtx));
 
   for (std::size_t opIndex = 2; opIndex < numOperands; ++opIndex) {
     if (std::optional<mlir::Value> isPresentRuntimeCheck =
@@ -122,10 +172,9 @@ lowerMinOrMax(fir::FirOpBuilder &builder, mlir::Location loc,
               .genThen([&]() {
                 llvm::SmallVector<fir::ExtendedValue> args;
                 args.emplace_back(extremum);
-                args.emplace_back(getOperand(opIndex));
-                fir::ExtendedValue newExtremum =
-                    Fortran::lower::genIntrinsicCall(builder, loc, name,
-                                                     resultType, args, stmtCtx);
+                args.emplace_back(getOperand(opIndex, loadOperand));
+                fir::ExtendedValue newExtremum = genIntrinsicCall(
+                    builder, loc, name, resultType, args, stmtCtx);
                 builder.create<fir::ResultOp>(loc, fir::getBase(newExtremum));
               })
               .genElse([&]() { builder.create<fir::ResultOp>(loc, extremum); })
@@ -134,9 +183,9 @@ lowerMinOrMax(fir::FirOpBuilder &builder, mlir::Location loc,
       // Argument is know to be present at compile time.
       llvm::SmallVector<fir::ExtendedValue> args;
       args.emplace_back(extremum);
-      args.emplace_back(getOperand(opIndex));
-      extremum = fir::getBase(Fortran::lower::genIntrinsicCall(
-          builder, loc, name, resultType, args, stmtCtx));
+      args.emplace_back(getOperand(opIndex, loadOperand));
+      extremum = fir::getBase(
+          genIntrinsicCall(builder, loc, name, resultType, args, stmtCtx));
     }
   }
   return extremum;
@@ -147,7 +196,7 @@ static void prepareIshftcArguments(
     const Fortran::evaluate::SpecificIntrinsic &intrinsic,
     std::optional<mlir::Type> retTy,
     const Fortran::lower::OperandPrepare &prepareOptionalArgument,
-    const Fortran::lower::OperandPrepare &prepareOtherArgument,
+    const Fortran::lower::OperandPrepareAs &prepareOtherArgument,
     Fortran::lower::AbstractConverter &converter) {
   for (auto arg : llvm::enumerate(procRef.arguments())) {
     const auto *expr =
@@ -160,7 +209,7 @@ static void prepareIshftcArguments(
       prepareOptionalArgument(*expr);
     } else {
       // Non optional arguments.
-      prepareOtherArgument(*expr);
+      prepareOtherArgument(*expr, fir::LowerIntrinsicArgAs::Value);
     }
   }
 }
@@ -178,15 +227,16 @@ lowerIshftc(fir::FirOpBuilder &builder, mlir::Location loc,
   assert(retTy && "ISFHTC must have a return type");
   mlir::Type resultType = *retTy;
   llvm::SmallVector<fir::ExtendedValue> args;
-  args.push_back(getOperand(0));
-  args.push_back(getOperand(1));
+  const bool loadOperand = true;
+  args.push_back(getOperand(0, loadOperand));
+  args.push_back(getOperand(1, loadOperand));
   auto iPC = isPresentCheck(2);
   assert(iPC.has_value());
   args.push_back(builder
                      .genIfOp(loc, {resultType}, *iPC,
                               /*withElseRegion=*/true)
                      .genThen([&]() {
-                       fir::ExtendedValue sizeExv = getOperand(2);
+                       fir::ExtendedValue sizeExv = getOperand(2, loadOperand);
                        mlir::Value size = builder.createConvert(
                            loc, resultType, fir::getBase(sizeExv));
                        builder.create<fir::ResultOp>(loc, size);
@@ -198,8 +248,64 @@ lowerIshftc(fir::FirOpBuilder &builder, mlir::Location loc,
                        builder.create<fir::ResultOp>(loc, bitSize);
                      })
                      .getResults()[0]);
-  return Fortran::lower::genIntrinsicCall(builder, loc, name, resultType, args,
-                                          stmtCtx);
+  return genIntrinsicCall(builder, loc, name, resultType, args, stmtCtx);
+}
+
+static void prepareAssociatedArguments(
+    const Fortran::evaluate::ProcedureRef &procRef,
+    const Fortran::evaluate::SpecificIntrinsic &intrinsic,
+    std::optional<mlir::Type> retTy,
+    const Fortran::lower::OperandPrepare &prepareOptionalArgument,
+    const Fortran::lower::OperandPrepareAs &prepareOtherArgument,
+    Fortran::lower::AbstractConverter &converter) {
+  const auto *pointer = procRef.UnwrapArgExpr(0);
+  const auto *optionalTarget = procRef.UnwrapArgExpr(1);
+  assert(pointer && optionalTarget &&
+         "expected call to associated with a target");
+  prepareOtherArgument(*pointer, fir::LowerIntrinsicArgAs::Inquired);
+  prepareOptionalArgument(*optionalTarget);
+}
+
+static fir::ExtendedValue
+lowerAssociated(fir::FirOpBuilder &builder, mlir::Location loc,
+                llvm::StringRef name, std::optional<mlir::Type> resultType,
+                const Fortran::lower::OperandPresent &isPresentCheck,
+                const Fortran::lower::OperandGetter &getOperand,
+                std::size_t numOperands,
+                Fortran::lower::StatementContext &stmtCtx) {
+  assert(numOperands == 2 && "expect two arguments when TARGET is OPTIONAL");
+  llvm::SmallVector<fir::ExtendedValue> args;
+  args.push_back(getOperand(0, /*loadOperand=*/false));
+  // Ensure a null descriptor is passed to the code lowering Associated if
+  // TARGET is absent.
+  fir::ExtendedValue targetExv = getOperand(1, /*loadOperand=*/false);
+  mlir::Value targetBase = fir::getBase(targetExv);
+  // subtle: isPresentCheck would test for an unallocated/disassociated target,
+  // while the optionality of the target pointer/allocatable is what must be
+  // checked here.
+  mlir::Value isPresent =
+      builder.create<fir::IsPresentOp>(loc, builder.getI1Type(), targetBase);
+  mlir::Type targetType = fir::unwrapRefType(targetBase.getType());
+  mlir::Type targetValueType = fir::unwrapPassByRefType(targetType);
+  mlir::Type boxType = targetType.isa<fir::BaseBoxType>()
+                           ? targetType
+                           : fir::BoxType::get(targetValueType);
+  fir::BoxValue targetBox =
+      builder
+          .genIfOp(loc, {boxType}, isPresent,
+                   /*withElseRegion=*/true)
+          .genThen([&]() {
+            mlir::Value box = builder.createBox(loc, targetExv);
+            mlir::Value cast = builder.createConvert(loc, boxType, box);
+            builder.create<fir::ResultOp>(loc, cast);
+          })
+          .genElse([&]() {
+            mlir::Value absentBox = builder.create<fir::AbsentOp>(loc, boxType);
+            builder.create<fir::ResultOp>(loc, absentBox);
+          })
+          .getResults()[0];
+  args.emplace_back(std::move(targetBox));
+  return genIntrinsicCall(builder, loc, name, resultType, args, stmtCtx);
 }
 
 void Fortran::lower::prepareCustomIntrinsicArgument(
@@ -207,12 +313,17 @@ void Fortran::lower::prepareCustomIntrinsicArgument(
     const Fortran::evaluate::SpecificIntrinsic &intrinsic,
     std::optional<mlir::Type> retTy,
     const OperandPrepare &prepareOptionalArgument,
-    const OperandPrepare &prepareOtherArgument, AbstractConverter &converter) {
+    const OperandPrepareAs &prepareOtherArgument,
+    AbstractConverter &converter) {
   llvm::StringRef name = intrinsic.name;
   if (name == "min" || name == "max")
     return prepareMinOrMaxArguments(procRef, intrinsic, retTy,
                                     prepareOptionalArgument,
                                     prepareOtherArgument, converter);
+  if (name == "associated")
+    return prepareAssociatedArguments(procRef, intrinsic, retTy,
+                                      prepareOptionalArgument,
+                                      prepareOtherArgument, converter);
   assert(name == "ishftc" && "unexpected custom intrinsic argument call");
   return prepareIshftcArguments(procRef, intrinsic, retTy,
                                 prepareOptionalArgument, prepareOtherArgument,
@@ -227,6 +338,9 @@ fir::ExtendedValue Fortran::lower::lowerCustomIntrinsic(
   if (name == "min" || name == "max")
     return lowerMinOrMax(builder, loc, name, retTy, isPresentCheck, getOperand,
                          numOperands, stmtCtx);
+  if (name == "associated")
+    return lowerAssociated(builder, loc, name, retTy, isPresentCheck,
+                           getOperand, numOperands, stmtCtx);
   assert(name == "ishftc" && "unexpected custom intrinsic call");
   return lowerIshftc(builder, loc, name, retTy, isPresentCheck, getOperand,
                      numOperands, stmtCtx);
