@@ -6,6 +6,7 @@
 //
 //===----------------------------------------------------------------------===//
 #include "check-acc-structure.h"
+#include "flang/Common/enum-set.h"
 #include "flang/Parser/parse-tree.h"
 #include "flang/Semantics/tools.h"
 
@@ -20,6 +21,35 @@
     RequiresConstantPositiveParameter(llvm::acc::Clause::Y, c.v); \
   }
 
+using ReductionOpsSet =
+    Fortran::common::EnumSet<Fortran::parser::AccReductionOperator::Operator,
+        Fortran::parser::AccReductionOperator::Operator_enumSize>;
+
+static ReductionOpsSet reductionIntegerSet{
+    Fortran::parser::AccReductionOperator::Operator::Plus,
+    Fortran::parser::AccReductionOperator::Operator::Multiply,
+    Fortran::parser::AccReductionOperator::Operator::Max,
+    Fortran::parser::AccReductionOperator::Operator::Min,
+    Fortran::parser::AccReductionOperator::Operator::Iand,
+    Fortran::parser::AccReductionOperator::Operator::Ior,
+    Fortran::parser::AccReductionOperator::Operator::Ieor};
+
+static ReductionOpsSet reductionRealSet{
+    Fortran::parser::AccReductionOperator::Operator::Plus,
+    Fortran::parser::AccReductionOperator::Operator::Multiply,
+    Fortran::parser::AccReductionOperator::Operator::Max,
+    Fortran::parser::AccReductionOperator::Operator::Min};
+
+static ReductionOpsSet reductionComplexSet{
+    Fortran::parser::AccReductionOperator::Operator::Plus,
+    Fortran::parser::AccReductionOperator::Operator::Multiply};
+
+static ReductionOpsSet reductionLogicalSet{
+    Fortran::parser::AccReductionOperator::Operator::And,
+    Fortran::parser::AccReductionOperator::Operator::Or,
+    Fortran::parser::AccReductionOperator::Operator::Eqv,
+    Fortran::parser::AccReductionOperator::Operator::Neqv};
+
 namespace Fortran::semantics {
 
 static constexpr inline AccClauseSet
@@ -307,8 +337,6 @@ void AccStructureChecker::Leave(const parser::OpenACCCacheConstruct &x) {
 }
 
 // Clause checkers
-CHECK_REQ_SCALAR_INT_CONSTANT_CLAUSE(Collapse, ACCC_collapse)
-
 CHECK_SIMPLE_CLAUSE(Auto, ACCC_auto)
 CHECK_SIMPLE_CLAUSE(Async, ACCC_async)
 CHECK_SIMPLE_CLAUSE(Attach, ACCC_attach)
@@ -326,7 +354,6 @@ CHECK_SIMPLE_CLAUSE(DeviceResident, ACCC_device_resident)
 CHECK_SIMPLE_CLAUSE(DeviceType, ACCC_device_type)
 CHECK_SIMPLE_CLAUSE(Finalize, ACCC_finalize)
 CHECK_SIMPLE_CLAUSE(Firstprivate, ACCC_firstprivate)
-CHECK_SIMPLE_CLAUSE(Gang, ACCC_gang)
 CHECK_SIMPLE_CLAUSE(Host, ACCC_host)
 CHECK_SIMPLE_CLAUSE(If, ACCC_if)
 CHECK_SIMPLE_CLAUSE(IfPresent, ACCC_if_present)
@@ -334,12 +361,10 @@ CHECK_SIMPLE_CLAUSE(Independent, ACCC_independent)
 CHECK_SIMPLE_CLAUSE(Link, ACCC_link)
 CHECK_SIMPLE_CLAUSE(NoCreate, ACCC_no_create)
 CHECK_SIMPLE_CLAUSE(Nohost, ACCC_nohost)
-CHECK_SIMPLE_CLAUSE(NumGangs, ACCC_num_gangs)
 CHECK_SIMPLE_CLAUSE(NumWorkers, ACCC_num_workers)
 CHECK_SIMPLE_CLAUSE(Present, ACCC_present)
 CHECK_SIMPLE_CLAUSE(Private, ACCC_private)
 CHECK_SIMPLE_CLAUSE(Read, ACCC_read)
-CHECK_SIMPLE_CLAUSE(Reduction, ACCC_reduction)
 CHECK_SIMPLE_CLAUSE(Seq, ACCC_seq)
 CHECK_SIMPLE_CLAUSE(Tile, ACCC_tile)
 CHECK_SIMPLE_CLAUSE(UseDevice, ACCC_use_device)
@@ -407,6 +432,85 @@ void AccStructureChecker::Enter(const parser::AccClause::Copyout &c) {
   }
 }
 
+void AccStructureChecker::Enter(const parser::AccClause::Gang &g) {
+  CheckAllowed(llvm::acc::Clause::ACCC_gang);
+
+  if (g.v) {
+    bool hasNum = false;
+    bool hasDim = false;
+    const Fortran::parser::AccGangArgList &x = *g.v;
+    for (const Fortran::parser::AccGangArg &gangArg : x.v) {
+      if (std::get_if<Fortran::parser::AccGangArg::Num>(&gangArg.u))
+        hasNum = true;
+      else if (std::get_if<Fortran::parser::AccGangArg::Dim>(&gangArg.u))
+        hasDim = true;
+    }
+
+    if (hasDim && hasNum)
+      context_.Say(GetContext().clauseSource,
+          "The num argument is not allowed when dim is specified"_err_en_US);
+  }
+}
+
+void AccStructureChecker::Enter(const parser::AccClause::NumGangs &n) {
+  CheckAllowed(llvm::acc::Clause::ACCC_num_gangs);
+
+  if (n.v.size() > 3)
+    context_.Say(GetContext().clauseSource,
+        "NUM_GANGS clause accepts a maximum of 3 arguments"_err_en_US);
+}
+
+void AccStructureChecker::Enter(const parser::AccClause::Reduction &reduction) {
+  CheckAllowed(llvm::acc::Clause::ACCC_reduction);
+
+  // From OpenACC 3.3
+  // At a minimum, the supported data types include Fortran logical as well as
+  // the numerical data types (e.g. integer, real, double precision, complex).
+  // However, for each reduction operator, the supported data types include only
+  // the types permitted as operands to the corresponding operator in the base
+  // language where (1) for max and min, the corresponding operator is less-than
+  // and (2) for other operators, the operands and the result are the same type.
+  //
+  // The following check that the reduction operator is supported with the given
+  // type.
+  const parser::AccObjectListWithReduction &list{reduction.v};
+  const auto &op{std::get<parser::AccReductionOperator>(list.t)};
+  const auto &objects{std::get<parser::AccObjectList>(list.t)};
+
+  for (const auto &object : objects.v) {
+    std::visit(
+        Fortran::common::visitors{
+            [&](const Fortran::parser::Designator &designator) {
+              if (const auto *name = getDesignatorNameIfDataRef(designator)) {
+                const auto *type{name->symbol->GetType()};
+                if (type->IsNumeric(TypeCategory::Integer) &&
+                    !reductionIntegerSet.test(op.v)) {
+                  context_.Say(GetContext().clauseSource,
+                      "reduction operator not supported for integer type"_err_en_US);
+                } else if (type->IsNumeric(TypeCategory::Real) &&
+                    !reductionRealSet.test(op.v)) {
+                  context_.Say(GetContext().clauseSource,
+                      "reduction operator not supported for real type"_err_en_US);
+                } else if (type->IsNumeric(TypeCategory::Complex) &&
+                    !reductionComplexSet.test(op.v)) {
+                  context_.Say(GetContext().clauseSource,
+                      "reduction operator not supported for complex type"_err_en_US);
+                } else if (type->category() ==
+                        Fortran::semantics::DeclTypeSpec::Category::Logical &&
+                    !reductionLogicalSet.test(op.v)) {
+                  context_.Say(GetContext().clauseSource,
+                      "reduction operator not supported for logical type"_err_en_US);
+                }
+                // TODO: check composite type.
+              }
+            },
+            [&](const Fortran::parser::Name &name) {
+              // TODO: check common block
+            }},
+        object.u);
+  }
+}
+
 void AccStructureChecker::Enter(const parser::AccClause::Self &x) {
   CheckAllowed(llvm::acc::Clause::ACCC_self);
   const std::optional<parser::AccSelfClause> &accSelfClause = x.v;
@@ -432,6 +536,15 @@ void AccStructureChecker::Enter(const parser::AccClause::Self &x) {
   }
 }
 
+void AccStructureChecker::Enter(const parser::AccClause::Collapse &x) {
+  CheckAllowed(llvm::acc::Clause::ACCC_collapse);
+  const parser::AccCollapseArg &accCollapseArg = x.v;
+  const auto &collapseValue{
+      std::get<parser::ScalarIntConstantExpr>(accCollapseArg.t)};
+  RequiresConstantPositiveParameter(
+      llvm::acc::Clause::ACCC_collapse, collapseValue);
+}
+
 llvm::StringRef AccStructureChecker::getClauseName(llvm::acc::Clause clause) {
   return llvm::acc::getOpenACCClauseName(clause);
 }
