@@ -21,7 +21,6 @@
 #include "terminator.h"
 #include "tools.h"
 #include "flang/Runtime/descriptor.h"
-#include <algorithm>
 
 namespace Fortran::runtime {
 
@@ -29,9 +28,9 @@ namespace Fortran::runtime {
 // for each of the vector sections of the result.
 class ShiftControl {
 public:
-  ShiftControl(const Descriptor &s, Terminator &t, int dim)
+  RT_API_ATTRS ShiftControl(const Descriptor &s, Terminator &t, int dim)
       : shift_{s}, terminator_{t}, shiftRank_{s.rank()}, dim_{dim} {}
-  void Init(const Descriptor &source, const char *which) {
+  RT_API_ATTRS void Init(const Descriptor &source, const char *which) {
     int rank{source.rank()};
     RUNTIME_CHECK(terminator_, shiftRank_ == 0 || shiftRank_ == rank - 1);
     auto catAndKind{shift_.type().GetCategoryAndKind()};
@@ -57,7 +56,7 @@ public:
           GetInt64(shift_.OffsetElement<char>(), shiftElemLen_, terminator_);
     }
   }
-  SubscriptValue GetShift(const SubscriptValue resultAt[]) const {
+  RT_API_ATTRS SubscriptValue GetShift(const SubscriptValue resultAt[]) const {
     if (shiftRank_ > 0) {
       SubscriptValue shiftAt[maxRank];
       int k{0};
@@ -85,7 +84,7 @@ private:
 };
 
 // Fill an EOSHIFT result with default boundary values
-static void DefaultInitialize(
+static RT_API_ATTRS void DefaultInitialize(
     const Descriptor &result, Terminator &terminator) {
   auto catAndKind{result.type().GetCategoryAndKind()};
   RUNTIME_CHECK(
@@ -95,14 +94,14 @@ static void DefaultInitialize(
   if (catAndKind->first == TypeCategory::Character) {
     switch (int kind{catAndKind->second}) {
     case 1:
-      std::fill_n(result.OffsetElement<char>(), bytes, ' ');
+      Fortran::runtime::fill_n(result.OffsetElement<char>(), bytes, ' ');
       break;
     case 2:
-      std::fill_n(result.OffsetElement<char16_t>(), bytes / 2,
+      Fortran::runtime::fill_n(result.OffsetElement<char16_t>(), bytes / 2,
           static_cast<char16_t>(' '));
       break;
     case 4:
-      std::fill_n(result.OffsetElement<char32_t>(), bytes / 4,
+      Fortran::runtime::fill_n(result.OffsetElement<char32_t>(), bytes / 4,
           static_cast<char32_t>(' '));
       break;
     default:
@@ -113,7 +112,7 @@ static void DefaultInitialize(
   }
 }
 
-static inline std::size_t AllocateResult(Descriptor &result,
+static inline RT_API_ATTRS std::size_t AllocateResult(Descriptor &result,
     const Descriptor &source, int rank, const SubscriptValue extent[],
     Terminator &terminator, const char *function) {
   std::size_t elementLen{source.ElementBytes()};
@@ -134,8 +133,8 @@ static inline std::size_t AllocateResult(Descriptor &result,
 }
 
 template <TypeCategory CAT, int KIND>
-static inline std::size_t AllocateBesselResult(Descriptor &result, int32_t n1,
-    int32_t n2, Terminator &terminator, const char *function) {
+static inline RT_API_ATTRS std::size_t AllocateBesselResult(Descriptor &result,
+    int32_t n1, int32_t n2, Terminator &terminator, const char *function) {
   int rank{1};
   SubscriptValue extent[maxRank];
   for (int j{0}; j < maxRank; j++) {
@@ -159,8 +158,8 @@ static inline std::size_t AllocateBesselResult(Descriptor &result, int32_t n1,
 }
 
 template <TypeCategory CAT, int KIND>
-static inline void DoBesselJn(Descriptor &result, int32_t n1, int32_t n2,
-    CppTypeFor<CAT, KIND> x, CppTypeFor<CAT, KIND> bn2,
+static inline RT_API_ATTRS void DoBesselJn(Descriptor &result, int32_t n1,
+    int32_t n2, CppTypeFor<CAT, KIND> x, CppTypeFor<CAT, KIND> bn2,
     CppTypeFor<CAT, KIND> bn2_1, const char *sourceFile, int line) {
   Terminator terminator{sourceFile, line};
   AllocateBesselResult<CAT, KIND>(result, n1, n2, terminator, "BESSEL_JN");
@@ -212,8 +211,8 @@ static inline void DoBesselJn(Descriptor &result, int32_t n1, int32_t n2,
 }
 
 template <TypeCategory CAT, int KIND>
-static inline void DoBesselJnX0(Descriptor &result, int32_t n1, int32_t n2,
-    const char *sourceFile, int line) {
+static inline RT_API_ATTRS void DoBesselJnX0(Descriptor &result, int32_t n1,
+    int32_t n2, const char *sourceFile, int line) {
   Terminator terminator{sourceFile, line};
   AllocateBesselResult<CAT, KIND>(result, n1, n2, terminator, "BESSEL_JN");
 
@@ -240,8 +239,8 @@ static inline void DoBesselJnX0(Descriptor &result, int32_t n1, int32_t n2,
 }
 
 template <TypeCategory CAT, int KIND>
-static inline void DoBesselYn(Descriptor &result, int32_t n1, int32_t n2,
-    CppTypeFor<CAT, KIND> x, CppTypeFor<CAT, KIND> bn1,
+static inline RT_API_ATTRS void DoBesselYn(Descriptor &result, int32_t n1,
+    int32_t n2, CppTypeFor<CAT, KIND> x, CppTypeFor<CAT, KIND> bn1,
     CppTypeFor<CAT, KIND> bn1_1, const char *sourceFile, int line) {
   Terminator terminator{sourceFile, line};
   AllocateBesselResult<CAT, KIND>(result, n1, n2, terminator, "BESSEL_YN");
@@ -293,8 +292,8 @@ static inline void DoBesselYn(Descriptor &result, int32_t n1, int32_t n2,
 }
 
 template <TypeCategory CAT, int KIND>
-static inline void DoBesselYnX0(Descriptor &result, int32_t n1, int32_t n2,
-    const char *sourceFile, int line) {
+static inline RT_API_ATTRS void DoBesselYnX0(Descriptor &result, int32_t n1,
+    int32_t n2, const char *sourceFile, int line) {
   Terminator terminator{sourceFile, line};
   AllocateBesselResult<CAT, KIND>(result, n1, n2, terminator, "BESSEL_YN");
 
@@ -319,17 +318,18 @@ static inline void DoBesselYnX0(Descriptor &result, int32_t n1, int32_t n2,
 }
 
 extern "C" {
+RT_EXT_API_GROUP_BEGIN
 
 // BESSEL_JN
 // TODO: REAL(2 & 3)
-void RTNAME(BesselJn_4)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselJn_4)(Descriptor &result, int32_t n1, int32_t n2,
     CppTypeFor<TypeCategory::Real, 4> x, CppTypeFor<TypeCategory::Real, 4> bn2,
     CppTypeFor<TypeCategory::Real, 4> bn2_1, const char *sourceFile, int line) {
   DoBesselJn<TypeCategory::Real, 4>(
       result, n1, n2, x, bn2, bn2_1, sourceFile, line);
 }
 
-void RTNAME(BesselJn_8)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselJn_8)(Descriptor &result, int32_t n1, int32_t n2,
     CppTypeFor<TypeCategory::Real, 8> x, CppTypeFor<TypeCategory::Real, 8> bn2,
     CppTypeFor<TypeCategory::Real, 8> bn2_1, const char *sourceFile, int line) {
   DoBesselJn<TypeCategory::Real, 8>(
@@ -337,7 +337,7 @@ void RTNAME(BesselJn_8)(Descriptor &result, int32_t n1, int32_t n2,
 }
 
 #if LDBL_MANT_DIG == 64
-void RTNAME(BesselJn_10)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselJn_10)(Descriptor &result, int32_t n1, int32_t n2,
     CppTypeFor<TypeCategory::Real, 10> x,
     CppTypeFor<TypeCategory::Real, 10> bn2,
     CppTypeFor<TypeCategory::Real, 10> bn2_1, const char *sourceFile,
@@ -348,7 +348,7 @@ void RTNAME(BesselJn_10)(Descriptor &result, int32_t n1, int32_t n2,
 #endif
 
 #if LDBL_MANT_DIG == 113 || HAS_FLOAT128
-void RTNAME(BesselJn_16)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselJn_16)(Descriptor &result, int32_t n1, int32_t n2,
     CppTypeFor<TypeCategory::Real, 16> x,
     CppTypeFor<TypeCategory::Real, 16> bn2,
     CppTypeFor<TypeCategory::Real, 16> bn2_1, const char *sourceFile,
@@ -359,25 +359,25 @@ void RTNAME(BesselJn_16)(Descriptor &result, int32_t n1, int32_t n2,
 #endif
 
 // TODO: REAL(2 & 3)
-void RTNAME(BesselJnX0_4)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselJnX0_4)(Descriptor &result, int32_t n1, int32_t n2,
     const char *sourceFile, int line) {
   DoBesselJnX0<TypeCategory::Real, 4>(result, n1, n2, sourceFile, line);
 }
 
-void RTNAME(BesselJnX0_8)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselJnX0_8)(Descriptor &result, int32_t n1, int32_t n2,
     const char *sourceFile, int line) {
   DoBesselJnX0<TypeCategory::Real, 8>(result, n1, n2, sourceFile, line);
 }
 
 #if LDBL_MANT_DIG == 64
-void RTNAME(BesselJnX0_10)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselJnX0_10)(Descriptor &result, int32_t n1, int32_t n2,
     const char *sourceFile, int line) {
   DoBesselJnX0<TypeCategory::Real, 10>(result, n1, n2, sourceFile, line);
 }
 #endif
 
 #if LDBL_MANT_DIG == 113 || HAS_FLOAT128
-void RTNAME(BesselJnX0_16)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselJnX0_16)(Descriptor &result, int32_t n1, int32_t n2,
     const char *sourceFile, int line) {
   DoBesselJnX0<TypeCategory::Real, 16>(result, n1, n2, sourceFile, line);
 }
@@ -385,14 +385,14 @@ void RTNAME(BesselJnX0_16)(Descriptor &result, int32_t n1, int32_t n2,
 
 // BESSEL_YN
 // TODO: REAL(2 & 3)
-void RTNAME(BesselYn_4)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselYn_4)(Descriptor &result, int32_t n1, int32_t n2,
     CppTypeFor<TypeCategory::Real, 4> x, CppTypeFor<TypeCategory::Real, 4> bn1,
     CppTypeFor<TypeCategory::Real, 4> bn1_1, const char *sourceFile, int line) {
   DoBesselYn<TypeCategory::Real, 4>(
       result, n1, n2, x, bn1, bn1_1, sourceFile, line);
 }
 
-void RTNAME(BesselYn_8)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselYn_8)(Descriptor &result, int32_t n1, int32_t n2,
     CppTypeFor<TypeCategory::Real, 8> x, CppTypeFor<TypeCategory::Real, 8> bn1,
     CppTypeFor<TypeCategory::Real, 8> bn1_1, const char *sourceFile, int line) {
   DoBesselYn<TypeCategory::Real, 8>(
@@ -400,7 +400,7 @@ void RTNAME(BesselYn_8)(Descriptor &result, int32_t n1, int32_t n2,
 }
 
 #if LDBL_MANT_DIG == 64
-void RTNAME(BesselYn_10)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselYn_10)(Descriptor &result, int32_t n1, int32_t n2,
     CppTypeFor<TypeCategory::Real, 10> x,
     CppTypeFor<TypeCategory::Real, 10> bn1,
     CppTypeFor<TypeCategory::Real, 10> bn1_1, const char *sourceFile,
@@ -411,7 +411,7 @@ void RTNAME(BesselYn_10)(Descriptor &result, int32_t n1, int32_t n2,
 #endif
 
 #if LDBL_MANT_DIG == 113 || HAS_FLOAT128
-void RTNAME(BesselYn_16)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselYn_16)(Descriptor &result, int32_t n1, int32_t n2,
     CppTypeFor<TypeCategory::Real, 16> x,
     CppTypeFor<TypeCategory::Real, 16> bn1,
     CppTypeFor<TypeCategory::Real, 16> bn1_1, const char *sourceFile,
@@ -422,32 +422,32 @@ void RTNAME(BesselYn_16)(Descriptor &result, int32_t n1, int32_t n2,
 #endif
 
 // TODO: REAL(2 & 3)
-void RTNAME(BesselYnX0_4)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselYnX0_4)(Descriptor &result, int32_t n1, int32_t n2,
     const char *sourceFile, int line) {
   DoBesselYnX0<TypeCategory::Real, 4>(result, n1, n2, sourceFile, line);
 }
 
-void RTNAME(BesselYnX0_8)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselYnX0_8)(Descriptor &result, int32_t n1, int32_t n2,
     const char *sourceFile, int line) {
   DoBesselYnX0<TypeCategory::Real, 8>(result, n1, n2, sourceFile, line);
 }
 
 #if LDBL_MANT_DIG == 64
-void RTNAME(BesselYnX0_10)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselYnX0_10)(Descriptor &result, int32_t n1, int32_t n2,
     const char *sourceFile, int line) {
   DoBesselYnX0<TypeCategory::Real, 10>(result, n1, n2, sourceFile, line);
 }
 #endif
 
 #if LDBL_MANT_DIG == 113 || HAS_FLOAT128
-void RTNAME(BesselYnX0_16)(Descriptor &result, int32_t n1, int32_t n2,
+void RTDEF(BesselYnX0_16)(Descriptor &result, int32_t n1, int32_t n2,
     const char *sourceFile, int line) {
   DoBesselYnX0<TypeCategory::Real, 16>(result, n1, n2, sourceFile, line);
 }
 #endif
 
 // CSHIFT where rank of ARRAY argument > 1
-void RTNAME(Cshift)(Descriptor &result, const Descriptor &source,
+void RTDEF(Cshift)(Descriptor &result, const Descriptor &source,
     const Descriptor &shift, int dim, const char *sourceFile, int line) {
   Terminator terminator{sourceFile, line};
   int rank{source.rank()};
@@ -492,7 +492,7 @@ void RTNAME(Cshift)(Descriptor &result, const Descriptor &source,
 }
 
 // CSHIFT where rank of ARRAY argument == 1
-void RTNAME(CshiftVector)(Descriptor &result, const Descriptor &source,
+void RTDEF(CshiftVector)(Descriptor &result, const Descriptor &source,
     std::int64_t shift, const char *sourceFile, int line) {
   Terminator terminator{sourceFile, line};
   RUNTIME_CHECK(terminator, source.rank() == 1);
@@ -511,7 +511,7 @@ void RTNAME(CshiftVector)(Descriptor &result, const Descriptor &source,
 }
 
 // EOSHIFT of rank > 1
-void RTNAME(Eoshift)(Descriptor &result, const Descriptor &source,
+void RTDEF(Eoshift)(Descriptor &result, const Descriptor &source,
     const Descriptor &shift, const Descriptor *boundary, int dim,
     const char *sourceFile, int line) {
   Terminator terminator{sourceFile, line};
@@ -591,7 +591,7 @@ void RTNAME(Eoshift)(Descriptor &result, const Descriptor &source,
 }
 
 // EOSHIFT of vector
-void RTNAME(EoshiftVector)(Descriptor &result, const Descriptor &source,
+void RTDEF(EoshiftVector)(Descriptor &result, const Descriptor &source,
     std::int64_t shift, const Descriptor *boundary, const char *sourceFile,
     int line) {
   Terminator terminator{sourceFile, line};
@@ -623,7 +623,7 @@ void RTNAME(EoshiftVector)(Descriptor &result, const Descriptor &source,
 }
 
 // PACK
-void RTNAME(Pack)(Descriptor &result, const Descriptor &source,
+void RTDEF(Pack)(Descriptor &result, const Descriptor &source,
     const Descriptor &mask, const Descriptor *vector, const char *sourceFile,
     int line) {
   Terminator terminator{sourceFile, line};
@@ -697,7 +697,7 @@ void RTNAME(Pack)(Descriptor &result, const Descriptor &source,
 
 // RESHAPE
 // F2018 16.9.163
-void RTNAME(Reshape)(Descriptor &result, const Descriptor &source,
+void RTDEF(Reshape)(Descriptor &result, const Descriptor &source,
     const Descriptor &shape, const Descriptor *pad, const Descriptor *order,
     const char *sourceFile, int line) {
   // Compute and check the rank of the result.
@@ -804,7 +804,7 @@ void RTNAME(Reshape)(Descriptor &result, const Descriptor &source,
 }
 
 // SPREAD
-void RTNAME(Spread)(Descriptor &result, const Descriptor &source, int dim,
+void RTDEF(Spread)(Descriptor &result, const Descriptor &source, int dim,
     std::int64_t ncopies, const char *sourceFile, int line) {
   Terminator terminator{sourceFile, line};
   int rank{source.rank() + 1};
@@ -838,7 +838,7 @@ void RTNAME(Spread)(Descriptor &result, const Descriptor &source, int dim,
 }
 
 // TRANSPOSE
-void RTNAME(Transpose)(Descriptor &result, const Descriptor &matrix,
+void RTDEF(Transpose)(Descriptor &result, const Descriptor &matrix,
     const char *sourceFile, int line) {
   Terminator terminator{sourceFile, line};
   RUNTIME_CHECK(terminator, matrix.rank() == 2);
@@ -857,7 +857,7 @@ void RTNAME(Transpose)(Descriptor &result, const Descriptor &matrix,
 }
 
 // UNPACK
-void RTNAME(Unpack)(Descriptor &result, const Descriptor &vector,
+void RTDEF(Unpack)(Descriptor &result, const Descriptor &vector,
     const Descriptor &mask, const Descriptor &field, const char *sourceFile,
     int line) {
   Terminator terminator{sourceFile, line};
@@ -903,5 +903,6 @@ void RTNAME(Unpack)(Descriptor &result, const Descriptor &vector,
   }
 }
 
+RT_EXT_API_GROUP_END
 } // extern "C"
 } // namespace Fortran::runtime
