@@ -32,18 +32,16 @@ test(S s)
     assert(s.front() == typename S::value_type('z'));
 }
 
+template <class S>
+TEST_CONSTEXPR_CXX20 void test_string() {
+  test(S("1"));
+  test(S("1234567890123456789012345678901234567890"));
+}
+
 TEST_CONSTEXPR_CXX20 bool test() {
-  {
-    typedef std::string S;
-    test(S("1"));
-    test(S("1234567890123456789012345678901234567890"));
-  }
+  test_string<std::string>();
 #if TEST_STD_VER >= 11
-  {
-    typedef std::basic_string<char, std::char_traits<char>, min_allocator<char>> S;
-    test(S("1"));
-    test(S("1234567890123456789012345678901234567890"));
-  }
+  test_string<std::basic_string<char, std::char_traits<char>, min_allocator<char>>>();
 #endif
 
   return true;
